@@ -35,17 +35,24 @@ import yaml
 from packaging import version
 
 from concurrencytest import ConcurrentTestSuite, fork_for_tests
-# pylint: disable=import-error
 from mininet.log import setLogLevel
 from mininet.clean import Cleanup
 
-import mininet_test_util
+from clib import mininet_test_util
+
+DEFAULT_HARDWARE = 'Open vSwitch'
 
 # Only these hardware types will be tested with meters.
 SUPPORTS_METERS = (
     'Aruba',
     'NoviFlow',
-    'Open vSwitch',
+# TODO: troubleshoot meters in OVS 2.10.0
+#   DEFAULT_HARDWARE,
+    'ZodiacGX',
+)
+
+SUPPORTS_METADATA = (
+    DEFAULT_HARDWARE,
 )
 
 
@@ -121,7 +128,7 @@ def import_hw_config():
             'of_port': (int,),
             'gauge_of_port': (int,),
         }
-        for required_key, required_key_types in list(required_config.items()):
+        for required_key, required_key_types in required_config.items():
             if required_key not in config:
                 print('%s must be specified in %s to use HW switch.' % (
                     required_key, config_file_name))
@@ -143,8 +150,7 @@ def import_hw_config():
                   '%d are provided in %s.' %
                   (REQUIRED_TEST_PORTS, len(dp_ports), config_file_name))
         return config
-    else:
-        return None
+    return None
 
 
 def check_dependencies():
@@ -326,25 +332,28 @@ def pipeline_superset_report(decoded_pcap_logs):
 def filter_test_hardware(test_obj, hw_config):
     test_hosts = test_obj.N_TAGGED + test_obj.N_UNTAGGED + test_obj.N_EXTENDED
     test_links = test_hosts * test_obj.LINKS_PER_HOST
-    if hw_config is not None:
+    testing_hardware = hw_config is not None
+    test_hardware = DEFAULT_HARDWARE
+    if testing_hardware:
         test_hardware = hw_config['hardware']
+
+    if test_obj.REQUIRES_METERS and test_hardware not in SUPPORTS_METERS:
+        return False
+
+    if test_obj.REQUIRES_METADATA and test_hardware not in SUPPORTS_METADATA:
+        return False
+
+    if testing_hardware:
         if test_obj.SOFTWARE_ONLY:
             return False
         if test_obj.NUM_DPS > 1:
             # TODO: test other stacking combinations.
             if test_obj.NUM_HOSTS > 2:
                 return False
-        else:
-            if test_links < REQUIRED_TEST_PORTS:
-                if test_hardware == 'ZodiacFX':
-                    return True
-                return False
-        if test_obj.REQUIRES_METERS:
-            if test_hardware not in SUPPORTS_METERS:
-                return False
-    else:
-        if test_obj.NUM_DPS == 1 and test_links < REQUIRED_TEST_PORTS:
-            return False
+
+    if test_obj.NUM_DPS == 1 and test_links < REQUIRED_TEST_PORTS:
+        return False
+
     return True
 
 
@@ -375,7 +384,7 @@ def expand_tests(module, requested_test_classes, excluded_test_classes,
             if test_name.startswith('FaucetSanity'):
                 sanity_test_suites.append(test_suite)
             else:
-                if serial or test_name.startswith('FaucetSingle'):
+                if serial or test_name.startswith('FaucetSingle') or test_obj.NETNS:
                     single_test_suites.append(test_suite)
                 else:
                     parallel_test_suites.append(test_suite)
