@@ -14,7 +14,7 @@ from mininet.log import error, debug
 from mininet.node import Host
 from mininet.util import quietRun, errRun
 
-from mininet_test_util import DEVNULL
+from clib.mininet_test_util import DEVNULL
 
 DEFAULT_NETWORK = 'none'
 DEFAULT_PREFIX = 'mininet'
@@ -210,7 +210,7 @@ class DockerHost(Host):
     def wait(self):
         """Wait for an activated container to terminate."""
         try:
-            if self.active_pipe_returncode != None:
+            if self.active_pipe_returncode is not None:
                 return self.active_pipe_returncode
             debug('Waiting for container %s.' % self.container)
             assert self.active_pipe, "container not activated"
@@ -309,5 +309,6 @@ def make_docker_host(image, prefix=DEFAULT_PREFIX, network=DEFAULT_NETWORK,
                 env_val = os.environ['DOCKER_STARTUP_TIMEOUT_MS']
                 if env_val:
                     kwargs['startup_timeout_ms'] = int(env_val)
-            DockerHost.__init__(self, *args, **kwargs)
+            super(_ImageHost, self).__init__(*args, **kwargs)
+
     return _ImageHost
