@@ -25,7 +25,7 @@ echo "  FAUCET_TESTS=\"$FAUCET_TESTS\""
 echo "try:"
 echo "  docker/runtests.sh"
 echo "or:"
-echo '  cd tests && ./faucet_mininet_test.py $FAUCET_TESTS'
+echo '  cd tests && ./run_integration_tests.sh $FAUCET_TESTS'
 echo '  cd clib && ./clib_mininet_test.py $FAUCET_TESTS'
 echo
 
