@@ -21,9 +21,8 @@ import netaddr
 from faucet import valve_of
 from faucet import valve_acl
 from faucet.valve_of import MATCH_FIELDS, OLD_MATCH_FIELDS
-from faucet.faucet_pipeline import ValveTableConfig
-from faucet.valve_table import ValveTable, ValveGroupTable
 from faucet.conf import Conf, test_config_condition, InvalidConfigError
+from faucet.valve_table import wildcard_table
 
 
 class ACL(Conf):
@@ -96,12 +95,14 @@ The output action contains a dictionary with the following elements:
         'vlan_vids': list,
     }
 
-    wildcard_table = ValveTable(
-        valve_of.ofp.OFPTT_ALL, 'all', ValveTableConfig('all'), flow_cookie=0)
-
     def __init__(self, _id, dp_id, conf):
         self.rules = []
         self.exact_match = None
+        self.meter = False
+        self.matches = {}
+        self.set_fields = set()
+        for match_fields in (MATCH_FIELDS, OLD_MATCH_FIELDS):
+            self.rule_types.update({match: (str, int) for match in match_fields.keys()})
         conf = copy.deepcopy(conf)
         if isinstance(conf, dict):
             rules = conf.get('rules', [])
@@ -110,61 +111,57 @@ The output action contains a dictionary with the following elements:
             conf = {}
         else:
             raise InvalidConfigError(
-                'ACL conf is an invalid type %s' % self._id)
+                'ACL conf is an invalid type %s' % _id)
         conf['rules'] = []
         for rule in rules:
-            test_config_condition(not isinstance(rule, dict), (
-                'ACL rule is %s not %s' % (type(rule), dict)))
-            if 'rule' in rule:
-                conf['rules'].append(rule['rule'])
-            else:
-                conf['rules'].append(rule)
+            normalized_rule = rule
+            if isinstance(rule, dict):
+                normalized_rule = rule.get('rule', rule)
+                if normalized_rule is None:
+                    normalized_rule = {k: v for k, v in rule.items() if v is not None}
+            test_config_condition(not isinstance(normalized_rule, dict), (
+                'ACL rule is %s not %s (%s)' % (type(normalized_rule), dict, rules)))
+            conf['rules'].append(normalized_rule)
         super(ACL, self).__init__(_id, dp_id, conf)
 
     def check_config(self):
         test_config_condition(
             not self.rules, 'no rules found for ACL %s' % self._id)
-        for match_fields in (MATCH_FIELDS, OLD_MATCH_FIELDS):
-            for match in list(match_fields.keys()):
-                self.rule_types[match] = (str, int)
         for rule in self.rules:
             self._check_conf_types(rule, self.rule_types)
-            for rule_field, rule_conf in list(rule.items()):
+            for rule_field, rule_conf in rule.items():
                 if rule_field == 'cookie':
-                    test_config_condition(rule_conf < 0 or rule_conf > 2**16, (
-                        'rule cookie value must be 0-2**16'))
+                    test_config_condition(
+                        rule_conf < 0 or rule_conf > 2**16,
+                        'rule cookie value must be 0-2**16')
                 elif rule_field == 'actions':
                     test_config_condition(
                         not rule_conf,
-                        'Missing rule actions in ACL %s' % self._id
-                        )
+                        'Missing rule actions in ACL %s' % self._id)
                     self._check_conf_types(rule_conf, self.actions_types)
-                    for action_name, action_conf in list(rule_conf.items()):
+                    for action_name, action_conf in rule_conf.items():
                         if action_name == 'output':
                             self._check_conf_types(
                                 action_conf, self.output_actions_types)
 
-    def to_conf(self):
-        return [{'rule': rule} for rule in self.rules]
-
-    def build(self, vid, meters):
+    def build(self, meters, vid, port_num):
         """Check that ACL can be built from config."""
 
         class NullRyuDatapath:
             """Placeholder Ryu Datapath."""
             ofproto = valve_of.ofp
 
-        matches = {}
-        set_fields = set()
-        meter = False
+        self.matches = {}
+        self.set_fields = set()
+        self.meter = False
         if self.rules:
             try:
                 ofmsgs = valve_acl.build_acl_ofmsgs(
-                    [self], self.wildcard_table,
-                    valve_of.goto_table(self.wildcard_table),
-                    valve_of.goto_table(self.wildcard_table),
+                    [self], wildcard_table,
+                    valve_of.goto_table(wildcard_table),
+                    valve_of.goto_table(wildcard_table),
                     2**16-1, meters, self.exact_match,
-                    vlan_vid=vid)
+                    vlan_vid=vid, port_num=port_num)
             except (netaddr.core.AddrFormatError, KeyError, ValueError) as err:
                 raise InvalidConfigError(err)
             test_config_condition(not ofmsgs, 'OF messages is empty')
@@ -184,15 +181,15 @@ The output action contains a dictionary with the following elements:
                         if valve_of.is_apply_actions(inst):
                             apply_actions.extend(inst.actions)
                         elif valve_of.is_meter(inst):
-                            meter = True
+                            self.meter = True
                     for action in apply_actions:
                         if valve_of.is_set_field(action):
-                            set_fields.add(action.key)
-                    for match, value in list(ofmsg.match.items()):
+                            self.set_fields.add(action.key)
+                    for match, value in ofmsg.match.items():
                         has_mask = isinstance(value, (tuple, list))
-                        if has_mask or match not in matches:
-                            matches[match] = has_mask
-        return (matches, set_fields, meter)
+                        if has_mask or match not in self.matches:
+                            self.matches[match] = has_mask
+        return (self.matches, self.set_fields, self.meter)
 
     def get_meters(self):
         for rule in self.rules:
@@ -208,7 +205,7 @@ The output action contains a dictionary with the following elements:
 
     def _resolve_output_ports(self, action_conf, resolve_port_cb):
         result = {}
-        for output_action, output_action_values in list(action_conf.items()):
+        for output_action, output_action_values in action_conf.items():
             if output_action == 'port':
                 port_name = output_action_values
                 port = resolve_port_cb(port_name)
@@ -232,7 +229,7 @@ The output action contains a dictionary with the following elements:
                 test_config_condition(not isinstance(failover, dict), (
                     'failover is not a dictionary'))
                 result[output_action] = {}
-                for failover_name, failover_values in list(failover.items()):
+                for failover_name, failover_values in failover.items():
                     if failover_name == 'ports':
                         resolved_ports = [
                             resolve_port_cb(p) for p in failover_values]
@@ -255,7 +252,7 @@ The output action contains a dictionary with the following elements:
                 resolved_actions = {}
                 test_config_condition(not isinstance(actions_conf, dict), (
                     'actions value is not a dictionary'))
-                for action_name, action_conf in list(actions_conf.items()):
+                for action_name, action_conf in actions_conf.items():
                     if action_name == 'mirror':
                         resolved_port = resolve_port_cb(action_conf)
                         test_config_condition(
@@ -271,3 +268,11 @@ The output action contains a dictionary with the following elements:
                     else:
                         resolved_actions[action_name] = action_conf
                 rule_conf['actions'] = resolved_actions
+
+
+# TODO: 802.1x steals the port ACL table.
+PORT_ACL_8021X = ACL(
+    'port_acl_8021x', 0,
+    {'rules': [{'eth_type': 1, 'eth_src': '01:02:03:04:05:06', 'actions': {
+        'output': {'set_fields': [{'eth_src': '01:02:03:04:05:06'}, {'eth_dst': '01:02:03:04:05:06'}]}}}]})
+PORT_ACL_8021X.build({}, None, 1)
