@@ -27,6 +27,8 @@ from ryu.lib import hub # pylint: disable=wrong-import-position
 
 from beka.beka import Beka # pylint: disable=wrong-import-position
 
+from faucet.valve_util import kill_on_exception
+
 
 class BgpSpeakerKey:
     """Uniquely describe a BGP speaker."""
@@ -53,14 +55,26 @@ class BgpSpeakerKey:
 class FaucetBgp:
     """Wrapper for Ryu BGP speaker."""
 
-    def __init__(self, logger, metrics, send_flow_msgs):
+    exc_logname = None
+
+    def __init__(self, logger, exc_logname, metrics, send_flow_msgs):
         self.logger = logger
+        self.exc_logname = exc_logname
         self.metrics = metrics
         self._send_flow_msgs = send_flow_msgs
         self._dp_bgp_speakers = {}
         self._dp_bgp_rib = {}
         self._valves = None
 
+    def _valve_vlan(self, dp_id, vlan_vid):
+        valve = None
+        vlan = None
+        if dp_id in self._valves:
+            if vlan_vid in self._valves[dp_id].dp.vlans:
+                valve = self._valves[dp_id]
+                vlan = valve.dp.vlans[vlan_vid]
+        return (valve, vlan)
+
     @staticmethod
     def _neighbor_states(bgp_speaker):
         """Return state of each neighbor for a BGP speaker as a list."""
@@ -69,22 +83,16 @@ class FaucetBgp:
             neighbor_states = bgp_speaker.neighbor_states()
         return neighbor_states
 
+    @kill_on_exception(exc_logname)
     def _bgp_up_handler(self, remote_ip, remote_as):
         self.logger.info('BGP peer router ID %s AS %s up' % (remote_ip, remote_as))
 
+    @kill_on_exception(exc_logname)
     def _bgp_down_handler(self, remote_ip, remote_as):
         self.logger.info('BGP peer router ID %s AS %s down' % (remote_ip, remote_as))
         # TODO: delete RIB routes for down peer.
 
-    def _valve_vlan(self, dp_id, vlan_vid):
-        valve = None
-        vlan = None
-        if dp_id in self._valves:
-            if vlan_vid in self._valves[dp_id].dp.vlans:
-                valve = self._valves[dp_id]
-                vlan = self._valves[dp_id].dp.vlans[vlan_vid]
-        return (valve, vlan)
-
+    @kill_on_exception(exc_logname)
     def _bgp_route_handler(self, path_change, bgp_speaker_key):
         """Handle a BGP change event.
 
@@ -108,11 +116,6 @@ class FaucetBgp:
                     'BGP nexthop %s for prefix %s cannot be us',
                     nexthop, prefix)
                 return
-            if vlan.ip_in_vip_subnet(nexthop) is None:
-                self.logger.error(
-                    'BGP nexthop %s for prefix %s is not a connected network',
-                    nexthop, prefix)
-                return
 
         flowmods = []
         if path_change.is_withdraw:
@@ -133,9 +136,8 @@ class FaucetBgp:
     def _bgp_vlans(valves):
         bgp_vlans = set()
         if valves:
-            for valve in list(valves.values()):
-                for vlan in valve.dp.bgp_vlans():
-                    bgp_vlans.add(vlan)
+            for valve in valves.values():
+                bgp_vlans.update({vlan for vlan in valve.dp.bgp_vlans()})
         return bgp_vlans
 
     @staticmethod
@@ -144,7 +146,7 @@ class FaucetBgp:
         for faucet_vip in vlan.faucet_vips_by_ipv(ipv):
             vlan_prefixes.append((str(faucet_vip), str(faucet_vip.ip)))
         routes = vlan.routes_by_ipv(ipv)
-        for ip_dst, ip_gw in list(routes.items()):
+        for ip_dst, ip_gw in routes.items():
             vlan_prefixes.append((str(ip_dst), str(ip_gw)))
         return vlan_prefixes
 
@@ -179,7 +181,7 @@ class FaucetBgp:
 
     def shutdown_bgp_speakers(self):
         """Shutdown any active BGP speakers."""
-        for bgp_speaker in list(self._dp_bgp_speakers.values()):
+        for bgp_speaker in self._dp_bgp_speakers.values():
             bgp_speaker.shutdown()
         self._dp_bgp_speakers = {}
 
@@ -200,7 +202,7 @@ class FaucetBgp:
                     bgp_speaker = self._dp_bgp_speakers[bgp_speaker_key]
                     if bgp_speaker_key in self._dp_bgp_rib:
                         # Re-add routes (to avoid flapping BGP even when VLAN cold starts).
-                        for prefix, nexthop in list(self._dp_bgp_rib[bgp_speaker_key].items()):
+                        for prefix, nexthop in self._dp_bgp_rib[bgp_speaker_key].items():
                             self.logger.info(
                                 'Re-adding %s via %s' % (prefix, nexthop))
                             flowmods = valve.add_route(bgp_vlan, nexthop, prefix)
@@ -211,7 +213,7 @@ class FaucetBgp:
                     bgp_speaker = self._create_bgp_speaker_for_vlan(bgp_vlan, bgp_speaker_key)
                 new_dp_bgp_speakers[bgp_speaker_key] = bgp_speaker
         # TODO: shutdown and remove deconfigured BGP speakers.
-        for bgp_speaker_key, old_bgp_speaker in list(self._dp_bgp_speakers.items()):
+        for bgp_speaker_key, old_bgp_speaker in self._dp_bgp_speakers.items():
             if bgp_speaker_key not in new_dp_bgp_speakers:
                 new_dp_bgp_speakers[bgp_speaker_key] = old_bgp_speaker
         self._dp_bgp_speakers = new_dp_bgp_speakers
@@ -219,7 +221,7 @@ class FaucetBgp:
 
     def update_metrics(self, _now):
         """Update BGP metrics."""
-        for bgp_speaker_key, bgp_speaker in list(self._dp_bgp_speakers.items()):
+        for bgp_speaker_key, bgp_speaker in self._dp_bgp_speakers.items():
             dp_id = bgp_speaker_key.dp_id
             vlan_vid = bgp_speaker_key.vlan_vid
             ipv = bgp_speaker_key.ipv
@@ -229,7 +231,7 @@ class FaucetBgp:
             neighbor_states = self._neighbor_states(bgp_speaker)
             for neighbor, neighbor_state in neighbor_states:
                 neighbor_labels = dict(
-                    valve.base_prom_labels, vlan=vlan.vid, neighbor=neighbor)
+                    valve.dp.base_prom_labels(), vlan=vlan.vid, neighbor=neighbor)
                 self.metrics.bgp_neighbor_uptime_seconds.labels( # pylint: disable=no-member
                     **neighbor_labels).set(neighbor_state['info']['uptime'])
                 self.metrics.bgp_neighbor_routes.labels( # pylint: disable=no-member
