@@ -21,7 +21,7 @@ from faucet import valve_of
 from faucet.conf import InvalidConfigError
 
 
-def push_vlan(vlan_vid):
+def push_vlan(acl_table, vlan_vid):
     """Push a VLAN tag with optional selection of eth type."""
     vid = vlan_vid
     vlan_eth_type = None
@@ -30,11 +30,12 @@ def push_vlan(vlan_vid):
         if 'eth_type' in vlan_vid:
             vlan_eth_type = vlan_vid['eth_type']
     if vlan_eth_type is None:
-        return valve_of.push_vlan_act(vid)
-    return valve_of.push_vlan_act(vid, eth_type=vlan_eth_type)
+        return valve_of.push_vlan_act(acl_table, vid)
+    return valve_of.push_vlan_act(
+        acl_table, vid, eth_type=vlan_eth_type)
 
 
-def rewrite_vlan(output_dict):
+def rewrite_vlan(acl_table, output_dict):
     """Implement actions to rewrite VLAN headers."""
     vlan_actions = []
     if 'pop_vlans' in output_dict:
@@ -42,30 +43,30 @@ def rewrite_vlan(output_dict):
             vlan_actions.append(valve_of.pop_vlan())
     # if vlan tag is specified, push it.
     if 'vlan_vid' in output_dict:
-        vlan_actions.extend(push_vlan(output_dict['vlan_vid']))
+        vlan_actions.extend(push_vlan(acl_table, output_dict['vlan_vid']))
     # swap existing VID
     elif 'swap_vid' in output_dict:
         vlan_actions.append(
-            valve_of.set_vlan_vid(output_dict['swap_vid']))
+            acl_table.set_vlan_vid(output_dict['swap_vid']))
     # or, if a list, push them all (all with type Q).
     elif 'vlan_vids' in output_dict:
         for vlan_vid in output_dict['vlan_vids']:
-            vlan_actions.extend(push_vlan(vlan_vid))
+            vlan_actions.extend(push_vlan(acl_table, vlan_vid))
     return vlan_actions
 
 
-def build_output_actions(output_dict):
+def build_output_actions(acl_table, output_dict):
     """Implement actions to alter packet/output."""
     output_actions = []
     output_port = None
     ofmsgs = []
     # rewrite any VLAN headers first always
-    vlan_actions = rewrite_vlan(output_dict)
+    vlan_actions = rewrite_vlan(acl_table, output_dict)
     if vlan_actions:
         output_actions.extend(vlan_actions)
     if 'set_fields' in output_dict:
-        for set_fields in output_dict['set_fields']:
-            output_actions.append(valve_of.set_field(**set_fields))
+        for set_field in output_dict['set_fields']:
+            output_actions.append(acl_table.set_field(**set_field))
     if 'port' in output_dict:
         output_port = output_dict['port']
         output_actions.append(valve_of.output_port(output_port))
@@ -87,9 +88,10 @@ def build_output_actions(output_dict):
 
 # TODO: change this, maybe this can be rewritten easily
 # possibly replace with a class for ACLs
-def build_acl_entry(rule_conf, meters,
+def build_acl_entry(acl_table, rule_conf, meters,
                     acl_allow_inst, acl_force_port_vlan_inst,
                     port_num=None, vlan_vid=None):
+    """Build flow/groupmods for one ACL rule entry."""
     acl_inst = []
     acl_act = []
     acl_match_dict = {}
@@ -97,7 +99,7 @@ def build_acl_entry(rule_conf, meters,
     acl_cookie = None
     allow_inst = acl_allow_inst
 
-    for attrib, attrib_value in list(rule_conf.items()):
+    for attrib, attrib_value in rule_conf.items():
         if attrib == 'in_port':
             continue
         if attrib == 'cookie':
@@ -125,7 +127,7 @@ def build_acl_entry(rule_conf, meters,
                     allow = True
             if 'output' in attrib_value:
                 output_port, output_actions, output_ofmsgs = build_output_actions(
-                    attrib_value['output'])
+                    acl_table, attrib_value['output'])
                 acl_act.extend(output_actions)
                 acl_ofmsgs.extend(output_ofmsgs)
 
@@ -154,12 +156,13 @@ def build_acl_ofmsgs(acls, acl_table,
                      acl_allow_inst, acl_force_port_vlan_inst,
                      highest_priority, meters,
                      exact_match, port_num=None, vlan_vid=None):
+    """Build flow/groupmods for all entries in an ACL."""
     ofmsgs = []
     acl_rule_priority = highest_priority
     for acl in acls:
         for rule_conf in acl.rules:
             acl_match, acl_inst, acl_cookie, acl_ofmsgs = build_acl_entry(
-                rule_conf, meters,
+                acl_table, rule_conf, meters,
                 acl_allow_inst, acl_force_port_vlan_inst,
                 port_num, vlan_vid)
             ofmsgs.extend(acl_ofmsgs)
