@@ -20,26 +20,33 @@
 import random
 
 from faucet import valve_of
+from faucet.faucet_metadata import get_egress_metadata
+from faucet.valve_manager_base import ValveManagerBase
 
 
-class ValveHostManager:
+class ValveHostManager(ValveManagerBase):
     """Manage host learning on VLANs."""
 
-    # don't update host cache more often than this many seconds
-    CACHE_UPDATE_GUARD_TIME = 2
-
     def __init__(self, logger, ports, vlans, eth_src_table, eth_dst_table,
-                 learn_timeout, learn_jitter, learn_ban_timeout, low_priority, host_priority):
+                 eth_dst_hairpin_table, pipeline, learn_timeout, learn_jitter,
+                 learn_ban_timeout, cache_update_guard_time, idle_dst):
         self.logger = logger
         self.ports = ports
         self.vlans = vlans
         self.eth_src_table = eth_src_table
         self.eth_dst_table = eth_dst_table
+        self.eth_dst_hairpin_table = eth_dst_hairpin_table
+        self.pipeline = pipeline
         self.learn_timeout = learn_timeout
         self.learn_jitter = learn_jitter
         self.learn_ban_timeout = learn_ban_timeout
-        self.low_priority = low_priority
-        self.host_priority = host_priority
+        self.low_priority = self._LOW_PRIORITY
+        self.host_priority = self._MATCH_PRIORITY
+        self.cache_update_guard_time = cache_update_guard_time
+        self.output_table = self.eth_dst_table
+        self.idle_dst = idle_dst
+        if self.eth_dst_hairpin_table:
+            self.output_table = self.eth_dst_hairpin_table
 
     def ban_rules(self, pkt_meta):
         """Limit learning to a maximum configured on this port/VLAN.
@@ -59,7 +66,8 @@ class ValveHostManager:
         if entry is None:
             if port.max_hosts:
                 if port.hosts_count() == port.max_hosts:
-                    ofmsgs.append(self._temp_ban_host_learning(self.eth_src_table.match(in_port=port.number)))
+                    ofmsgs.append(self._temp_ban_host_learning(
+                        self.eth_src_table.match(in_port=port.number)))
                     port.dyn_learn_ban_count += 1
                     self.logger.info(
                         'max hosts %u reached on %s, '
@@ -73,11 +81,41 @@ class ValveHostManager:
                     vlan.dyn_learn_ban_count += 1
                     self.logger.info(
                         'max hosts %u reached on VLAN %u, '
-                        'temporarily banning learning on this vlan, '
+                        'temporarily banning learning on this VLAN, '
                         'and not learning %s on %s' % (
                             vlan.max_hosts, vlan.vid, eth_src, port))
         return ofmsgs
 
+    def add_port(self, port):
+        """initialise override_output_port if necessary"""
+        ofmsgs = []
+        if port.override_output_port:
+            ofmsgs.append(self.eth_src_table.flowmod(
+                match=self.eth_src_table.match(
+                    in_port=port.number),
+                priority=self.low_priority + 1,
+                inst=[valve_of.apply_actions([
+                    valve_of.output_controller(),
+                    valve_of.output_port(port.override_output_port.number)])]))
+        return ofmsgs
+
+    def del_port(self, port):
+        ofmsgs = []
+        if port.permanent_learn:
+            for entry in port.hosts():
+                ofmsgs.extend(self.pipeline.remove_filter(
+                    {'eth_src': entry.eth_src}))
+        return ofmsgs
+
+    def initialise_tables(self):
+        ofmsgs = []
+        for vlan in self.vlans.values():
+            ofmsgs.append(self.eth_src_table.flowcontroller(
+                match=self.eth_src_table.match(vlan=vlan),
+                priority=self.low_priority,
+                inst=[self.eth_src_table.goto(self.output_table)]))
+        return ofmsgs
+
     def _temp_ban_host_learning(self, match):
         return self.eth_src_table.flowdrop(
             match,
@@ -86,12 +124,11 @@ class ValveHostManager:
 
     def delete_host_from_vlan(self, eth_src, vlan):
         """Delete a host from a VLAN."""
-        ofmsgs = []
-        ofmsgs.extend(self.eth_src_table.flowdel(
-            self.eth_src_table.match(vlan=vlan, eth_src=eth_src)))
-        ofmsgs.extend(self.eth_dst_table.flowdel(
-            self.eth_dst_table.match(vlan=vlan, eth_dst=eth_src)))
-        return ofmsgs
+        return [
+            self.eth_src_table.flowdel(
+                self.eth_src_table.match(vlan=vlan, eth_src=eth_src)),
+            self.eth_dst_table.flowdel(
+                self.eth_dst_table.match(vlan=vlan, eth_dst=eth_src))]
 
     def expire_hosts_from_vlan(self, vlan, now):
         """Expire hosts from VLAN cache."""
@@ -108,7 +145,7 @@ class ValveHostManager:
         return int(max(abs(
             self.learn_timeout -
             (self.learn_jitter / 2) + random.randint(0, self.learn_jitter)),
-                       self.CACHE_UPDATE_GUARD_TIME))
+                       self.cache_update_guard_time))
 
     def learn_host_timeouts(self, port):
         """Calculate flow timeouts for learning on a port."""
@@ -131,37 +168,28 @@ class ValveHostManager:
         # flows destined to controller
         src_rule_idle_timeout = 0
         src_rule_hard_timeout = learn_timeout
-        dst_rule_idle_timeout = learn_timeout
+        dst_rule_idle_timeout = learn_timeout + self.cache_update_guard_time
+        if not self.idle_dst:
+            dst_rule_idle_timeout = 0
         return (src_rule_idle_timeout, src_rule_hard_timeout, dst_rule_idle_timeout)
 
-    def learn_host_on_vlan_port_flows(self, port, vlan, eth_src, delete_existing,
-                                      src_rule_idle_timeout, src_rule_hard_timeout,
+    def learn_host_on_vlan_port_flows(self, port, vlan, eth_src,
+                                      delete_existing, refresh_rules,
+                                      src_rule_idle_timeout,
+                                      src_rule_hard_timeout,
                                       dst_rule_idle_timeout):
         """Return flows that implement learning a host on a port."""
         ofmsgs = []
 
-        if port.permanent_learn:
-            # Antispoofing rule for this MAC.
-            ofmsgs.append(self.eth_src_table.flowdrop(
-                self.eth_src_table.match(vlan=vlan, eth_src=eth_src),
-                priority=(self.host_priority - 2)))
-        else:
-            # Delete any existing entries for MAC.
-            if delete_existing:
-                ofmsgs.extend(self.delete_host_from_vlan(eth_src, vlan))
-
-        # Output packets for this MAC to specified port.
-        ofmsgs.append(self.eth_dst_table.flowmod(
-            self.eth_dst_table.match(vlan=vlan, eth_dst=eth_src),
-            priority=self.host_priority,
-            inst=[valve_of.apply_actions(vlan.output_port(port))],
-            idle_timeout=dst_rule_idle_timeout))
+        # Delete any existing entries for MAC.
+        if delete_existing:
+            ofmsgs.extend(self.delete_host_from_vlan(eth_src, vlan))
 
         # Associate this MAC with source port.
         src_match = self.eth_src_table.match(
             in_port=port.number, vlan=vlan, eth_src=eth_src)
         src_priority = self.host_priority - 1
-        inst = valve_of.goto_table(self.eth_dst_table)
+        inst = self.eth_src_table.goto(self.output_table)
 
         if port.override_output_port:
             inst = valve_of.apply_actions([
@@ -173,16 +201,28 @@ class ValveHostManager:
             hard_timeout=src_rule_hard_timeout,
             idle_timeout=src_rule_idle_timeout))
 
+        hairpinning = port.hairpin or port.hairpin_unicast
+
+        # If we are refreshing only and not in hairpin mode, leave existing eth_dst alone.
+        if refresh_rules and not hairpinning:
+            return ofmsgs
+
+        # Output packets for this MAC to specified port.
+        ofmsgs.append(self.eth_dst_table.flowmod(
+            self.eth_dst_table.match(vlan=vlan, eth_dst=eth_src),
+            priority=self.host_priority,
+            inst=self.pipeline.output(port, vlan),
+            idle_timeout=dst_rule_idle_timeout))
+
         # If port is in hairpin mode, install a special rule
         # that outputs packets destined to this MAC back out the same
         # port they came in (e.g. multiple hosts on same WiFi AP,
         # and FAUCET is switching between them on the same port).
-        if port.hairpin:
-            ofmsgs.append(self.eth_src_table.flowmod(
-                self.eth_src_table.match(in_port=port.number, vlan=vlan, eth_dst=eth_src),
-                priority=(self.host_priority + 1),
+        if hairpinning:
+            ofmsgs.append(self.eth_dst_hairpin_table.flowmod(
+                self.eth_dst_hairpin_table.match(in_port=port.number, vlan=vlan, eth_dst=eth_src),
+                priority=self.host_priority,
                 inst=[valve_of.apply_actions(vlan.output_port(port, hairpin=True))],
-                hard_timeout=src_rule_hard_timeout,
                 idle_timeout=dst_rule_idle_timeout))
 
         return ofmsgs
@@ -195,6 +235,8 @@ class ValveHostManager:
         cache_port = None
         cache_age = None
         entry = vlan.cached_host(eth_src)
+        refresh_rules = False
+
         # Host not cached, and no hosts expired since we cold started
         # Enable faster learning by assuming there's no previous host to delete
         if entry is None:
@@ -202,6 +244,11 @@ class ValveHostManager:
                     (vlan.dyn_last_time_hosts_expired is None or
                      vlan.dyn_last_time_hosts_expired < last_dp_coldstart_time)):
                 delete_existing = False
+        elif entry.port.permanent_learn:
+            if entry.port != port:
+                ofmsgs.extend(self.pipeline.filter_packets(
+                    {'eth_src': eth_src, 'in_port': port.number}))
+            return (ofmsgs, entry.port)
         else:
             cache_age = now - entry.cache_time
             cache_port = entry.port
@@ -210,11 +257,12 @@ class ValveHostManager:
             cache_port is not None and
             cache_port.lacp and port.lacp and cache_port.lacp == port.lacp)
         if cache_port == port or same_lag:
+            # if we very very recently learned this host, don't do anything.
+            if cache_age < self.cache_update_guard_time:
+                return (ofmsgs, cache_port)
             # skip delete if host didn't change ports or on same LAG.
             delete_existing = False
-            # if we very very recently learned this host, don't do anything.
-            if cache_age < self.CACHE_UPDATE_GUARD_TIME:
-                return (ofmsgs, cache_port)
+            refresh_rules = True
 
         if port.loop_protect:
             ban_age = None
@@ -224,12 +272,12 @@ class ValveHostManager:
             # prolong the ban
             if port.dyn_last_ban_time:
                 ban_age = now - port.dyn_last_ban_time
-                if ban_age < self.CACHE_UPDATE_GUARD_TIME:
+                if ban_age < self.cache_update_guard_time:
                     learn_ban = True
 
             # if not in protect mode and we get a rapid move, enact protect mode
             if not learn_ban and entry is not None:
-                if port != cache_port and cache_age < self.CACHE_UPDATE_GUARD_TIME:
+                if port != cache_port and cache_age < self.cache_update_guard_time:
                     learn_ban = True
                     port.dyn_learn_ban_count += 1
                     self.logger.info('rapid move of %s from %s to %s, temp loop ban %s' % (
@@ -238,7 +286,8 @@ class ValveHostManager:
             # already, or newly in protect mode, apply the ban rules.
             if learn_ban:
                 port.dyn_last_ban_time = now
-                ofmsgs.append(self._temp_ban_host_learning(self.eth_src_table.match(in_port=port.number)))
+                ofmsgs.append(self._temp_ban_host_learning(
+                    self.eth_src_table.match(in_port=port.number)))
                 return (ofmsgs, cache_port)
 
         (src_rule_idle_timeout,
@@ -246,7 +295,7 @@ class ValveHostManager:
          dst_rule_idle_timeout) = self.learn_host_timeouts(port)
 
         ofmsgs.extend(self.learn_host_on_vlan_port_flows(
-            port, vlan, eth_src, delete_existing,
+            port, vlan, eth_src, delete_existing, refresh_rules,
             src_rule_idle_timeout, src_rule_hard_timeout,
             dst_rule_idle_timeout))
 
@@ -274,7 +323,7 @@ class ValveHostFlowRemovedManager(ValveHostManager):
                 in_port = None
                 eth_src = None
                 eth_dst = None
-                for field, value in list(match.items()):
+                for field, value in match.items():
                     if field == 'in_port':
                         in_port = value
                     elif field == 'eth_src':
@@ -302,7 +351,7 @@ class ValveHostFlowRemovedManager(ValveHostManager):
         # Disable hard_time, dst rule expires after src rule.
         src_rule_idle_timeout = learn_timeout
         src_rule_hard_timeout = 0
-        dst_rule_idle_timeout = learn_timeout + self.CACHE_UPDATE_GUARD_TIME
+        dst_rule_idle_timeout = learn_timeout + self.cache_update_guard_time
         return (src_rule_idle_timeout, src_rule_hard_timeout, dst_rule_idle_timeout)
 
     def _src_rule_expire(self, vlan, port, eth_src):
