@@ -38,7 +38,10 @@ class ConfigWatcher:
         if self.config_hashes:
             new_config_file_stats = stat_config_files(self.config_hashes)
             if self.config_file_stats:
-                if new_config_file_stats != self.config_file_stats:
+                # Check content as well in case mtime et al was cached.
+                if new_config_file_stats == self.config_file_stats:
+                    changed = self.content_changed(self.config_file)
+                else:
                     changed = True
             self.config_file_stats = new_config_file_stats
         return changed
@@ -95,7 +98,7 @@ class ValvesManager:
     def new_valve(self, new_dp):
         valve_cl = valve_factory(new_dp)
         if valve_cl is not None:
-            return valve_cl(new_dp, self.logname, self.metrics, self.notifier)
+            return valve_cl(new_dp, self.logname, self.metrics, self.notifier, self.dot1x)
         self.logger.error(
             '%s hardware %s must be one of %s',
             new_dp.name,
@@ -106,70 +109,83 @@ class ValvesManager:
     def load_configs(self, now, new_config_file, delete_dp=None):
         """Load/apply new config to all Valves."""
         new_dps = self.parse_configs(new_config_file)
-        if new_dps is not None:
-            deleted_dpids = (
-                set(list(self.valves.keys())) -
-                set([dp.dp_id for dp in new_dps]))
-            for new_dp in new_dps:
-                dp_id = new_dp.dp_id
-                if dp_id in self.valves:
-                    self.logger.info('Reconfiguring existing datapath %s', dpid_log(dp_id))
-                    valve = self.valves[dp_id]
-                    ofmsgs = valve.reload_config(now, new_dp)
-                    if ofmsgs:
-                        self.send_flows_to_dp_by_id(valve, ofmsgs)
-                else:
-                    self.logger.info('Add new datapath %s', dpid_log(new_dp.dp_id))
-                    valve = self.new_valve(new_dp)
-                    if valve is None:
-                        continue
-                valve.update_config_metrics()
-                self.valves[dp_id] = valve
-            if delete_dp is not None:
-                for deleted_dp in deleted_dpids:
-                    delete_dp(deleted_dp)
-                    del self.valves[deleted_dp]
-            self.bgp.reset(self.valves)
-            self.dot1x.reset(self.valves)
+        if new_dps is None:
+            return False
+        deleted_dpids = (
+            set(self.valves.keys()) -
+            set([dp.dp_id for dp in new_dps]))
+        for new_dp in new_dps:
+            dp_id = new_dp.dp_id
+            if dp_id in self.valves:
+                self.logger.info('Reconfiguring existing datapath %s', dpid_log(dp_id))
+                valve = self.valves[dp_id]
+                ofmsgs = valve.reload_config(now, new_dp)
+                if ofmsgs:
+                    self.send_flows_to_dp_by_id(valve, ofmsgs)
+            else:
+                self.logger.info('Add new datapath %s', dpid_log(new_dp.dp_id))
+                valve = self.new_valve(new_dp)
+                if valve is None:
+                    continue
+            valve.update_config_metrics()
+            self.valves[dp_id] = valve
+        if delete_dp is not None:
+            for deleted_dp in deleted_dpids:
+                delete_dp(deleted_dp)
+                del self.valves[deleted_dp]
+        self.bgp.reset(self.valves)
+        self.dot1x.reset(self.valves)
+        return True
+
+    def _notify(self, event_dict):
+        """Send an event notification."""
+        self.notifier.notify(0, str(0), event_dict)
 
     def request_reload_configs(self, now, new_config_file, delete_dp=None):
         """Process a request to load config changes."""
         if self.config_watcher.content_changed(new_config_file):
             self.logger.info('configuration %s changed, analyzing differences', new_config_file)
-            self.load_configs(now, new_config_file, delete_dp=delete_dp)
+            result = self.load_configs(now, new_config_file, delete_dp=delete_dp)
+            self._notify({'CONFIG_CHANGE': {'success': result}})
         else:
             self.logger.info('configuration is unchanged, not reloading')
         self.metrics.faucet_config_reload_requests.inc() # pylint: disable=no-member
 
     def update_metrics(self, now):
         """Update metrics in all Valves."""
-        for valve in list(self.valves.values()):
+        for valve in self.valves.values():
             valve.update_metrics(now, rate_limited=False)
         self.bgp.update_metrics(now)
 
     def valve_flow_services(self, now, valve_service):
         """Call a method on all Valves and send any resulting flows."""
-        for valve in list(self.valves.values()):
+        for valve in self.valves.values():
             other_valves = self._other_running_valves(valve)
-            ofmsgs = getattr(valve, valve_service)(now, other_valves)
+            valve_service_labels = dict(valve.dp.base_prom_labels(), valve_service=valve_service)
+            valve_service_func = getattr(valve, valve_service)
+            with self.metrics.faucet_valve_service_secs.labels( # pylint: disable=no-member
+                    **valve_service_labels).time():
+                ofmsgs = valve_service_func(now, other_valves)
             if ofmsgs:
                 self.send_flows_to_dp_by_id(valve, ofmsgs)
 
     def _other_running_valves(self, valve):
-        return [other_valve for other_valve in list(self.valves.values())
+        return [other_valve for other_valve in self.valves.values()
                 if valve != other_valve and other_valve.dp.dyn_running]
 
     def valve_packet_in(self, now, valve, msg):
         """Time a call to Valve packet in handler."""
+        self.metrics.of_packet_ins.labels( # pylint: disable=no-member
+            **valve.dp.base_prom_labels()).inc()
         if valve.rate_limit_packet_ins(now):
             return
         pkt_meta = valve.parse_pkt_meta(msg)
         if pkt_meta is None:
+            self.metrics.of_unexpected_packet_ins.labels( # pylint: disable=no-member
+                **valve.dp.base_prom_labels()).inc()
             return
-        self.metrics.of_packet_ins.labels( # pylint: disable=no-member
-            **valve.base_prom_labels).inc()
         with self.metrics.faucet_packet_in_secs.labels( # pylint: disable=no-member
-                **valve.base_prom_labels).time():
+                **valve.dp.base_prom_labels()).time():
             ofmsgs = valve.rcv_packet(now, self._other_running_valves(valve), pkt_meta)
         if ofmsgs:
             self.send_flows_to_dp_by_id(valve, ofmsgs)
