@@ -19,12 +19,10 @@
 import collections
 import ipaddress
 import random
-
 import netaddr
 
 from faucet import valve_of
 from faucet.conf import Conf, test_config_condition, InvalidConfigError
-from faucet.valve_util import btos
 from faucet.valve_packet import FAUCET_MAC
 
 
@@ -110,7 +108,7 @@ class VLAN(Conf):
         # Don't proactively ARP for hosts if over this limit (default 2*max_hosts)
         'proactive_nd_limit': 0,
         # Don't proactively ND for hosts if over this limit (default 2*max_hosts)
-        'targeted_gw_resolution': False,
+        'targeted_gw_resolution': True,
         # If True, and a gateway has been resolved, target the first re-resolution attempt to the same port rather than flooding.
         'minimum_ip_size_check': True,
         # If False, don't check that IP packets have a payload (must be False for OVS trace/tutorial to work)
@@ -181,13 +179,17 @@ class VLAN(Conf):
 
         self.dyn_host_cache = None
         self.dyn_host_cache_by_port = None
+        self.dyn_host_cache_stats_stale = None
         self.dyn_last_time_hosts_expired = None
         self.dyn_learn_ban_count = 0
         self.dyn_neigh_cache_by_ipv = None
         self.dyn_oldest_host_time = None
+        self.dyn_last_updated_metrics_sec = None
 
         self.dyn_routes_by_ipv = collections.defaultdict(dict)
         self.dyn_gws_by_ipv = collections.defaultdict(dict)
+        self.dyn_host_gws_by_ipv = collections.defaultdict(set)
+        self.dyn_route_gws_by_ipv = collections.defaultdict(set)
         self.reset_caches()
         super(VLAN, self).__init__(_id, dp_id, conf)
 
@@ -203,7 +205,7 @@ class VLAN(Conf):
     @staticmethod
     def _check_ip_str(ip_str, ip_method=ipaddress.ip_address):
         try:
-            return ip_method(btos(ip_str))
+            return ip_method(ip_str)
         except (ValueError, AttributeError, TypeError) as err:
             raise InvalidConfigError('Invalid IP address %s: %s' % (ip_str, err))
 
@@ -249,7 +251,7 @@ class VLAN(Conf):
                 'BGP port must be %s not %s' % (int, type(self.bgp_port))))
             test_config_condition(self.bgp_connect_mode not in ('passive'), (
                 'BGP connect mode %s must be passive' % self.bgp_connect_mode))
-            test_config_condition(not ipaddress.IPv4Address(btos(self.bgp_routerid)), (
+            test_config_condition(not ipaddress.IPv4Address(self.bgp_routerid), (
                 '%s is not a valid IPv4 address' % (self.bgp_routerid)))
             test_config_condition(not self.bgp_neighbor_as, 'No BGP neighbor AS')
             test_config_condition(not self.bgp_neighbor_addresses, 'No BGP neighbor addresses')
@@ -286,15 +288,23 @@ class VLAN(Conf):
         """Reset dynamic caches."""
         self.dyn_host_cache = {}
         self.dyn_host_cache_by_port = {}
+        self.dyn_host_cache_stats_stale = {}
         self.dyn_neigh_cache_by_ipv = collections.defaultdict(dict)
+        self.dyn_unresolved_route_ip_gws = collections.defaultdict(list)
+        self.dyn_unresolved_host_ip_gws = collections.defaultdict(list)
 
     def reset_ports(self, ports):
-        self.tagged = tuple([port for port in ports if self in port.tagged_vlans])
-        self.untagged = tuple([port for port in ports if self == port.native_vlan])
+        """Reset tagged and untagged port lists."""
+        sorted_ports = sorted(ports, key=lambda i: i.number)
+        self.tagged = tuple([port for port in sorted_ports if self in port.tagged_vlans])
+        self.untagged = tuple([port for port in sorted_ports if self == port.native_vlan])
 
     def add_cache_host(self, eth_src, port, cache_time):
+        """Add/update a host to the cache on a port at at time."""
         existing_entry = self.cached_host(eth_src)
-        if existing_entry is not None:
+        if existing_entry is None:
+            self.dyn_host_cache_stats_stale[port.number] = True
+        else:
             self.dyn_host_cache_by_port[existing_entry.port.number].remove(
                 existing_entry)
         entry = HostCacheEntry(eth_src, port, cache_time)
@@ -307,6 +317,7 @@ class VLAN(Conf):
         """Expire a host from caches."""
         entry = self.cached_host(eth_src)
         if entry is not None:
+            self.dyn_host_cache_stats_stale[entry.port.number] = True
             self.dyn_host_cache_by_port[entry.port.number].remove(entry)
             del self.dyn_host_cache[eth_src]
 
@@ -324,9 +335,8 @@ class VLAN(Conf):
         return hosts_count
 
     def cached_host(self, eth_src):
-        if eth_src in self.dyn_host_cache:
-            return self.dyn_host_cache[eth_src]
-        return None
+        """Return host from cache or None."""
+        return self.dyn_host_cache.get(eth_src, None)
 
     def cached_host_on_port(self, eth_src, port):
         """Return host cache entry if host in cache and on specified port."""
@@ -338,22 +348,24 @@ class VLAN(Conf):
     def clear_cache_hosts_on_port(self, port):
         """Clear all hosts learned on a port."""
         for entry in self.cached_hosts_on_port(port):
+
             self.expire_cache_host(entry.eth_src)
 
     def expire_cache_hosts(self, now, learn_timeout):
         """Expire stale host entries."""
         expired_hosts = []
+        min_cache_time = now - learn_timeout
 
-        if self.dyn_oldest_host_time is None or now - self.dyn_oldest_host_time > learn_timeout:
-            min_cache_time = now - learn_timeout
-            self.dyn_oldest_host_time = now
-            for entry in list(self.dyn_host_cache.values()):
-                if (not entry.port.permanent_learn and entry.cache_time < min_cache_time):
-                    expired_hosts.append(entry)
-                else:
-                    self.dyn_oldest_host_time = min(entry.cache_time, self.dyn_oldest_host_time)
+        if self.dyn_oldest_host_time is None or self.dyn_oldest_host_time < min_cache_time:
+            expired_hosts = [
+                entry for entry in self.dyn_host_cache.values()
+                if entry.cache_time < min_cache_time and not entry.port.permanent_learn]
             for entry in expired_hosts:
                 self.expire_cache_host(entry.eth_src)
+            self.dyn_oldest_host_time = now
+            if self.dyn_host_cache:
+                self.dyn_oldest_host_time = min(
+                    [entry.cache_time for entry in self.dyn_host_cache.values()])
         return expired_hosts
 
     @staticmethod
@@ -369,6 +381,7 @@ class VLAN(Conf):
         return self._by_ipv(self.faucet_vips, ipv)
 
     def link_and_other_vips(self, ipv):
+        """Return link local and non-link local VIPs."""
         vips = self.faucet_vips_by_ipv(ipv)
         link_local_vips = frozenset([vip for vip in vips if vip.is_link_local])
         other_vips = vips - link_local_vips
@@ -413,12 +426,21 @@ class VLAN(Conf):
             return True
         return False
 
+    def _update_gw_types(self, ip_gw):
+        if self.is_host_fib_route(ip_gw):
+            self.dyn_host_gws_by_ipv[ip_gw.version].add(ip_gw)
+            self.dyn_route_gws_by_ipv[ip_gw.version] -= set([ip_gw])
+        else:
+            self.dyn_route_gws_by_ipv[ip_gw.version].add(ip_gw)
+            self.dyn_host_gws_by_ipv[ip_gw.version] -= set([ip_gw])
+
     def add_route(self, ip_dst, ip_gw):
         """Add an IP route."""
         self.dyn_routes_by_ipv[ip_gw.version][ip_dst] = ip_gw
         if ip_gw not in self.dyn_gws_by_ipv[ip_gw.version]:
             self.dyn_gws_by_ipv[ip_gw.version][ip_gw] = set()
         self.dyn_gws_by_ipv[ip_gw.version][ip_gw].add(ip_dst)
+        self._update_gw_types(ip_gw)
 
     def del_route(self, ip_dst):
         """Delete an IP route."""
@@ -427,6 +449,7 @@ class VLAN(Conf):
         self.dyn_gws_by_ipv[ip_gw.version][ip_gw].remove(ip_dst)
         if not self.dyn_gws_by_ipv[ip_gw.version][ip_gw]:
             del self.dyn_gws_by_ipv[ip_gw.version][ip_gw]
+        self._update_gw_types(ip_gw)
 
     def ip_dsts_for_ip_gw(self, ip_gw):
         """Return list of IP destinations, for specified gateway."""
@@ -451,9 +474,12 @@ class VLAN(Conf):
         return len(self.dyn_host_cache)
 
     def __str__(self):
-        port_list = tuple([str(x) for x in self.get_ports()])
-        ports = ','.join(port_list)
-        return 'VLAN %s vid:%s ports:%s' % (self.name, self.vid, ports)
+        str_ports = []
+        if self.tagged:
+            str_ports.append('tagged: %s' % ','.join([str(p) for p in self.tagged]))
+        if self.untagged:
+            str_ports.append('untagged: %s' % ','.join([str(p) for p in self.untagged]))
+        return 'VLAN %s vid:%s %s' % (self.name, self.vid, ' '.join(str_ports))
 
     def __repr__(self):
         return self.__str__()
@@ -467,17 +493,57 @@ class VLAN(Conf):
         return tuple([port for port in self.get_ports() if port.hairpin])
 
     def mirrored_ports(self):
-        """Return list of ports that are mirrored on this VLAN."""
+        """Return ports that are mirrored on this VLAN."""
         return tuple([port for port in self.get_ports() if port.mirror])
 
+    def loop_protect_external_ports(self):
+        """Return ports wth external loop protection set."""
+        return tuple([port for port in self.get_ports() if port.loop_protect_external])
+
+    def loop_protect_external_ports_up(self):
+        """Return up ports with external loop protection set."""
+        return tuple([port for port in self.loop_protect_external_ports() if port.dyn_phys_up])
+
+    def lacp_ports(self):
+        """Return ports that have LACP on this VLAN."""
+        return tuple([port for port in self.get_ports() if port.lacp])
+
+    def lacp_up_ports(self):
+        """Return ports that have LACP up on this VLAN."""
+        return tuple([port for port in self.lacp_ports() if port.dyn_lacp_up])
+
     def lags(self):
         """Return dict of LAGs mapped to member ports."""
-        lacp_ports = tuple([port for port in self.get_ports() if port.lacp])
         lags = collections.defaultdict(list)
-        for port in lacp_ports:
+        for port in self.lacp_ports():
             lags[port.lacp].append(port)
         return lags
 
+    def lags_up(self):
+        """Return dict of LAGs mapped to member ports that have LACP up."""
+        lags = collections.defaultdict(list)
+        for port in self.lacp_up_ports():
+            lags[port.lacp].append(port)
+        return lags
+
+    def exclude_same_lag_member_ports(self, in_port=None):
+        """Ensure output on only one member of a LAG."""
+        exclude_ports = set()
+        lags = self.lags()
+        if lags:
+            lags_up = self.lags_up()
+            if in_port is not None and in_port.lacp:
+                # Don't flood from one LACP bundle member, to another.
+                exclude_ports.update(lags[in_port.lacp])
+            # Pick one up bundle member to flood to.
+            for lag, ports in lags.items():
+                ports_up = lags_up[lag]
+                if ports_up:
+                    exclude_ports.update(ports[1:])
+                else:
+                    exclude_ports.update(ports)
+        return exclude_ports
+
     @staticmethod
     def flood_ports(configured_ports, exclude_unicast):
         if exclude_unicast:
@@ -507,17 +573,24 @@ class VLAN(Conf):
         pkt = packet_builder(vid, *args)
         return valve_of.packetout(port.number, pkt.data)
 
-    def flood_pkt(self, packet_builder, random_order, *args):
+    def flood_pkt(self, packet_builder, multi_out=True, *args):
         ofmsgs = []
         for vid, ports in (
                 (self.vid, self.tagged_flood_ports(False)),
                 (None, self.untagged_flood_ports(False))):
             if ports:
                 pkt = packet_builder(vid, *args)
-                flood_ofmsgs = [valve_of.packetout(port.number, pkt.data) for port in ports if port.running()]
-                if random_order:
-                    random.shuffle(flood_ofmsgs)
-                ofmsgs.extend(flood_ofmsgs)
+                exclude_ports = self.exclude_same_lag_member_ports()
+                running_ports = [
+                    port for port in ports if port.running() and port not in exclude_ports]
+                random.shuffle(running_ports)
+                if multi_out:
+                    ofmsgs.append(valve_of.packetouts(
+                        [port.number for port in running_ports], pkt.data))
+                else:
+                    ofmsgs.extend(
+                        [valve_of.packetout(port.number, pkt.data)
+                         for port in running_ports])
         return ofmsgs
 
     def port_is_tagged(self, port):
