@@ -19,8 +19,9 @@
 # limitations under the License.
 
 
+import os
 from copy import deepcopy
-from faucet.conf import Conf
+from faucet.conf import Conf, test_config_condition
 
 
 class WatcherConf(Conf):
@@ -76,17 +77,11 @@ For Prometheus:
        Defaults to '127.0.0.1'.
 """
 
-    defaults = {
-        'name': None,
+    db_defaults = {
         'type': None,
-        'dps': None,
-        'all_dps': False,
-        'interval': 30,
-        'db': None,
-        'dbs': None,
-        'db_type': 'text',
         'file': None,
-        'compress': False,   # compress flow table file
+        'compress': False,
+        # compress flow table file
         'influx_db': 'faucet',
         # influx database name
         'influx_host': 'localhost',
@@ -106,15 +101,8 @@ For Prometheus:
         'prometheus_test_thread': False,
     }
 
-    defaults_types = {
-        'name': str,
+    db_defaults_types = {
         'type': str,
-        'dps': list,
-        'all_dps': bool,
-        'interval': int,
-        'db': str,
-        'dbs': list,
-        'db_type': str,
         'file': str,
         'compress': bool,
         'influx_db': str,
@@ -129,6 +117,28 @@ For Prometheus:
         'prometheus_test_thread': bool,
     }
 
+    defaults = {
+        'name': None,
+        'type': None,
+        'dps': None,
+        'all_dps': False,
+        'interval': 30,
+        'db': None,
+        'dbs': None,
+        'db_type': 'text',
+    }
+
+    defaults_types = {
+        'name': str,
+        'type': str,
+        'dps': list,
+        'all_dps': bool,
+        'interval': int,
+        'db': str,
+        'dbs': list,
+        'db_type': str,
+    }
+
     def __init__(self, _id, dp_id, conf, prom_client):
         self.db = None # pylint: disable=invalid-name
         self.dbs = None
@@ -151,17 +161,30 @@ For Prometheus:
         self.prometheus_port = None
         self.prometheus_addr = None
         self.prometheus_test_thread = None
+        self.defaults.update(self.db_defaults)
+        self.defaults_types.update(self.db_defaults_types)
         super(WatcherConf, self).__init__(_id, dp_id, conf)
         self.name = str(self._id)
         self.prom_client = prom_client
 
     def add_db(self, db_conf):
         """Add database config to this watcher."""
+        self._check_conf_types(db_conf, self.db_defaults_types)
         db_conf = deepcopy(db_conf)
         db_type = db_conf.pop('type')
         db_conf['db_type'] = db_type
         self.update(db_conf)
+        test_config_condition(
+            self.file is not None and not
+            (os.path.dirname(self.file) and os.access(os.path.dirname(self.file), os.W_OK)),
+            '%s is not writable' % self.file)
 
     def add_dp(self, dp): # pylint: disable=invalid-name
         """Add a datapath to this watcher."""
         self.dp = dp # pylint: disable=invalid-name
+
+    def check_config(self):
+        super(WatcherConf, self).check_config()
+        test_config_condition(
+            self.all_dps and self.dps is not None,
+            'all_dps and dps cannot be set together')
