@@ -86,16 +86,15 @@ class FakeOFTable:
             # entries which will cause ambiguous behaviour. This is
             # obviously unnacceptable so we will assume this is
             # always set
-            add = True
             for fte in table:
                 if flowmod.fte_matches(fte, strict=True):
                     table.remove(fte)
                     break
                 elif flowmod.overlaps(fte):
-                    add = False
-                    break
-            if add:
-                table.append(flowmod)
+                    raise FakeOFTableException(
+                        'Overlapping flowmods {} and {}'.format(
+                            flowmod, fte))
+            table.append(flowmod)
 
         def _del(table, flowmod):
             removals = [fte for fte in table if flowmod.fte_matches(fte)]
@@ -205,6 +204,12 @@ class FakeOFTable:
                         for action in instruction.actions:
                             if action.type == ofp.OFPAT_SET_FIELD:
                                 packet_dict[action.key] = action.value
+                    elif instruction.type == ofp.OFPIT_WRITE_METADATA:
+                        metadata = packet_dict.get('metadata', 0)
+                        mask = instruction.metadata_mask
+                        mask_compl = mask ^ 0xFFFFFFFFFFFFFFFF
+                        packet_dict['metadata'] = (metadata & mask_compl)\
+                            | (instruction.metadata & mask)
         return instructions
 
     def is_output(self, match, port=None, vid=None):
@@ -305,6 +310,7 @@ class FlowMod:
         """flowmod is a ryu flow modification message object"""
         self.priority = flowmod.priority
         self.instructions = flowmod.instructions
+        self.validate_instructions()
         self.match_values = {}
         self.match_masks = {}
         self.out_port = None
@@ -323,6 +329,15 @@ class FlowMod:
             self.match_values[key] = val
             self.match_masks[key] = mask
 
+    def validate_instructions(self):
+        instruction_types = set()
+        for instruction in self.instructions:
+            if instruction.type in instruction_types:
+                raise FakeOFTableException(
+                    'FlowMod with Multiple instructions of the '
+                    'same type: {}'.format(self.instructions))
+            instruction_types.add(instruction.type)
+
     def out_port_matches(self, other):
         """returns True if other has an output action to this flowmods
         output_port"""
