@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-"""Unit tests run as PYTHONPATH=.. python3 ./test_valve.py."""
+"""Unit tests run as PYTHONPATH=../../.. python3 ./test_valve.py."""
 
 # Copyright (C) 2015 Research and Innovation Advanced Network New Zealand Ltd.
 # Copyright (C) 2015--2018 The Contributors
@@ -62,12 +62,13 @@ from fakeoftable import FakeOFTable
 
 FAUCET_MAC = '0e:00:00:00:00:01'
 
+
 # TODO: fix fake OF table implementation for in_port filtering
 # (ie. do not output to in_port)
-DP1_CONFIG = """
+BASE_DP1_CONFIG = """
         dp_id: 1
+        egress_pipeline: True
         ignore_learn_ins: 100
-        combinatorial_port_flood: True
         ofchannel_log: '/dev/null'
         packetin_pps: 99
         lldp_beacon:
@@ -75,13 +76,27 @@ DP1_CONFIG = """
             max_per_interval: 1
 """
 
+DP1_CONFIG = """
+        combinatorial_port_flood: True
+""" + BASE_DP1_CONFIG
+
 IDLE_DP1_CONFIG = """
         use_idle_timeout: True
 """ + DP1_CONFIG
 
 GROUP_DP1_CONFIG = """
         group_table: True
-""" + DP1_CONFIG
+        combinatorial_port_flood: False
+""" + BASE_DP1_CONFIG
+
+DOT1X_CONFIG = """
+        dot1x:
+            nfv_intf: lo
+            nfv_sw_port: 2
+            radius_ip: 127.0.0.1
+            radius_port: 1234
+            radius_secret: SECRET
+""" + BASE_DP1_CONFIG
 
 CONFIG = """
 dps:
@@ -142,6 +157,7 @@ dps:
                 number: 4
                 native_vlan: v300
             5:
+                description: p5
                 stack:
                     dp: s4
                     port: 5
@@ -162,6 +178,7 @@ dps:
                 number: 4
                 native_vlan: v300
             5:
+                description: p5
                 number: 5
                 stack:
                     dp: s3
@@ -214,9 +231,7 @@ def build_pkt(pkt):
     ethertype = None
     if 'arp_source_ip' in pkt and 'arp_target_ip' in pkt:
         ethertype = ether.ETH_TYPE_ARP
-        arp_code = arp.ARP_REQUEST
-        if pkt['eth_dst'] == FAUCET_MAC:
-            arp_code = arp.ARP_REPLY
+        arp_code = pkt.get('arp_code', arp.ARP_REQUEST)
         layers.append(arp.arp(
             src_ip=pkt['arp_source_ip'], dst_ip=pkt['arp_target_ip'], opcode=arp_code))
     elif 'ipv6_src' in pkt and 'ipv6_dst' in pkt:
@@ -314,7 +329,7 @@ class ValveTestBases:
         DP = 's1'
         DP_ID = 1
         NUM_PORTS = 5
-        NUM_TABLES = 9
+        NUM_TABLES = 10
         P1_V100_MAC = '00:00:00:01:00:01'
         P2_V200_MAC = '00:00:00:02:00:02'
         P3_V200_MAC = '00:00:00:02:00:03'
@@ -356,7 +371,8 @@ class ValveTestBases:
             # TODO: verify events
             self.notifier = faucet_experimental_event.FaucetExperimentalEventNotifier(
                 self.faucet_event_sock, self.metrics, self.logger)
-            self.bgp = faucet_bgp.FaucetBgp(self.logger, self.metrics, self.send_flows_to_dp_by_id)
+            self.bgp = faucet_bgp.FaucetBgp(
+                self.logger, logfile, self.metrics, self.send_flows_to_dp_by_id)
             self.dot1x = faucet_dot1x.FaucetDot1x(
                 self.logger, self.metrics, self.send_flows_to_dp_by_id)
             self.valves_manager = valves_manager.ValvesManager(
@@ -364,7 +380,7 @@ class ValveTestBases:
                 self.bgp, self.dot1x, self.send_flows_to_dp_by_id)
             self.last_flows_to_dp[self.DP_ID] = []
             self.notifier.start()
-            self.update_config(config)
+            self.update_config(config, reload_expected=False)
             self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             self.sock.connect(self.faucet_event_sock)
             self.connect_dp()
@@ -381,16 +397,18 @@ class ValveTestBases:
         def tearDown(self):
             self.teardown_valve()
 
-        def profile(self, func, sortby='cumulative', amount=20, count=1):
-            pr = cProfile.Profile()
-            pr.enable()
+        @staticmethod
+        def profile(func, sortby='cumulative', amount=20, count=1):
+            """Convenience method to profile a function call."""
+            prof = cProfile.Profile()
+            prof.enable()
             for _ in range(count):
-              func()
-            pr.disable()
-            pr_stream = io.StringIO()
-            ps = pstats.Stats(pr, stream=pr_stream).sort_stats(sortby)
-            ps.print_stats(amount)
-            print(pr_stream.getvalue())
+                func()
+            prof.disable()
+            prof_stream = io.StringIO()
+            prof_stats = pstats.Stats(prof, stream=prof_stream).sort_stats(sortby)
+            prof_stats.print_stats(amount)
+            print(prof_stream.getvalue())
 
         def get_prom(self, var, labels=None):
             """Return a Prometheus variable value."""
@@ -404,11 +422,16 @@ class ValveTestBases:
                 val = 0
             return val
 
-        def prom_inc(self, func, var, labels=None):
+        def prom_inc(self, func, var, labels=None, inc_expected=True):
             """Check Prometheus variable increments by 1 after calling a function."""
             before = self.get_prom(var, labels)
             func()
-            self.assertTrue(before + 1, self.get_prom(var, labels))
+            after = self.get_prom(var, labels)
+            msg = '%s %s before %f after %f' % (var, labels, before, after)
+            if inc_expected:
+                self.assertEqual(before + 1, after, msg=msg)
+            else:
+                self.assertEqual(before, after, msg=msg)
 
         def send_flows_to_dp_by_id(self, valve, flows):
             """Callback for ValvesManager to simulate sending flows to DP."""
@@ -416,45 +439,68 @@ class ValveTestBases:
             prepared_flows = valve.prepare_send_flows(flows)
             self.last_flows_to_dp[valve.dp.dp_id] = prepared_flows
 
-        def update_config(self, config):
+        def update_config(self, config, reload_type='cold', reload_expected=True):
             """Update FAUCET config with config as text."""
-            self.assertFalse(self.valves_manager.config_watcher.files_changed())
-            existing_config = os.path.exists(self.config_file)
+            before_dp_status = int(self.get_prom('dp_status'))
+            existing_config = None
+            if os.path.exists(self.config_file):
+                with open(self.config_file) as config_file:
+                    existing_config = config_file.read()
             with open(self.config_file, 'w') as config_file:
                 config_file.write(config)
-            if existing_config:
-                self.assertTrue(self.valves_manager.config_watcher.files_changed())
+            content_change_expected = config != existing_config
+            self.assertEqual(
+                content_change_expected,
+                self.valves_manager.config_watcher.content_changed(self.config_file))
             self.last_flows_to_dp[self.DP_ID] = []
-            self.valves_manager.request_reload_configs(time.time(), self.config_file)
+            var = 'faucet_config_reload_%s_total' % reload_type
+            self.prom_inc(
+                partial(self.valves_manager.request_reload_configs,
+                        time.time(), self.config_file), var=var, inc_expected=reload_expected)
             self.valve = self.valves_manager.valves[self.DP_ID]
             if self.DP_ID in self.last_flows_to_dp:
                 reload_ofmsgs = self.last_flows_to_dp[self.DP_ID]
                 self.table.apply_ofmsgs(reload_ofmsgs)
+            self.assertEqual(before_dp_status, int(self.get_prom('dp_status')))
 
         def connect_dp(self):
             """Call DP connect and set all ports to up."""
+            self.assertEqual(0, int(self.get_prom('dp_status')))
             discovered_up_ports = [port_no for port_no in range(1, self.NUM_PORTS + 1)]
             self.table.apply_ofmsgs(
                 self.valve.switch_features(None) +
                 self.valve.datapath_connect(time.time(), discovered_up_ports))
             self.assertEqual(1, int(self.get_prom('dp_status')))
             for port_no in discovered_up_ports:
-                self.set_port_up(port_no)
+                if port_no in self.valve.dp.ports:
+                    self.set_port_up(port_no)
             self.assertTrue(self.valve.dp.to_conf())
 
+        def port_labels(self, port_no):
+            port = self.valve.dp.ports[port_no]
+            return {'port': port.name, 'port_description': port.description}
+
+        def port_expected_status(self, port_no, exp_status):
+            if port_no not in self.valve.dp.ports:
+                return
+            labels = self.port_labels(port_no)
+            status = int(self.get_prom('port_status', labels=labels))
+            self.assertEqual(
+                status, exp_status,
+                msg='status %u != expected %u for port %s' % (
+                    status, exp_status, labels))
+
         def set_port_down(self, port_no):
             """Set port status of port to down."""
             self.table.apply_ofmsgs(self.valve.port_status_handler(
                 port_no, ofp.OFPPR_DELETE, ofp.OFPPS_LINK_DOWN))
-            self.assertEqual(
-                0, int(self.get_prom('port_status', labels={'port': str(port_no)})))
+            self.port_expected_status(port_no, 0)
 
         def set_port_up(self, port_no):
             """Set port status of port to up."""
             self.table.apply_ofmsgs(self.valve.port_status_handler(
                 port_no, ofp.OFPPR_ADD, 0))
-            self.assertEqual(
-                1, int(self.get_prom('port_status', labels={'port': str(port_no)})))
+            self.port_expected_status(port_no, 1)
 
         def flap_port(self, port_no):
             """Flap op status on a port."""
@@ -547,7 +593,7 @@ class ValveTestBases:
                         output = _verify_flood_to_port(match, port, valve_vlan)
                         if port == in_port:
                             self.assertNotEqual(
-                                combinatorial_port_flood, output,
+                                output, combinatorial_port_flood,
                                 msg=('flooding to in_port (%s) not '
                                      'compatible with flood mode (%s)') % (
                                          output, combinatorial_port_flood))
@@ -588,12 +634,11 @@ class ValveTestBases:
             now = time.time()
             self.prom_inc(
                 partial(self.valves_manager.valve_packet_in, now, self.valve, msg),
-                'of_packet_ins')
+                'of_packet_ins_total')
             rcv_packet_ofmsgs = self.last_flows_to_dp[self.DP_ID]
             self.table.apply_ofmsgs(rcv_packet_ofmsgs)
             for valve_service in (
-                    'resolve_gateways', 'advertise',
-                    'send_lldp_beacons', 'state_expire'):
+                    'resolve_gateways', 'advertise', 'fast_advertise', 'state_expire'):
                 self.valves_manager.valve_flow_services(
                     now, valve_service)
             self.valves_manager.update_metrics(now)
@@ -624,9 +669,20 @@ class ValveTestBases:
         def test_disconnect(self):
             """Test disconnection of DP from controller."""
             self.assertEqual(1, int(self.get_prom('dp_status')))
-            self.prom_inc(partial(self.valve.datapath_disconnect), 'of_dp_disconnections')
+            self.prom_inc(partial(self.valve.datapath_disconnect), 'of_dp_disconnections_total')
             self.assertEqual(0, int(self.get_prom('dp_status')))
 
+        def test_unexpected_port(self):
+            """Test packet in from unexpected port."""
+            self.prom_inc(
+                partial(self.rcv_packet, 999, 0x100, {
+                    'eth_src': self.P1_V300_MAC,
+                    'eth_dst': self.UNKNOWN_MAC,
+                    'ipv4_src': '10.0.0.1',
+                    'ipv4_dst': '10.0.0.2'}),
+                'of_unexpected_packet_ins_total',
+                inc_expected=True)
+
         def test_oferror(self):
             """Test OFError handler."""
             datapath = None
@@ -685,21 +741,36 @@ class ValveTestBases:
                 'chassis_id': self.P1_V100_MAC,
                 'port_id': 1}))
 
-        def test_arp_for_controller(self):
-            """ARP request for controller VIP."""
-            arp_replies = self.rcv_packet(1, 0x100, {
+        def test_bogon_arp_for_controller(self):
+            """Bogon ARP request for controller VIP."""
+            replies = self.rcv_packet(1, 0x100, {
                 'eth_src': self.P1_V100_MAC,
                 'eth_dst': mac.BROADCAST_STR,
-                'arp_source_ip': '10.0.0.1',
+                'arp_code': arp.ARP_REQUEST,
+                'arp_source_ip': '8.8.8.8',
                 'arp_target_ip': '10.0.0.254'})
-            # TODO: check ARP reply is valid
-            self.assertTrue(self.packet_outs_from_flows(arp_replies))
+            # Must be no ARP reply to an ARP request not in our subnet.
+            self.assertFalse(self.packet_outs_from_flows(replies))
+
+        def test_arp_for_controller(self):
+            """ARP request for controller VIP."""
+            for _retries in range(3):
+                for arp_mac in (mac.BROADCAST_STR, self.valve.dp.vlans[0x100].faucet_mac):
+                    arp_replies = self.rcv_packet(1, 0x100, {
+                        'eth_src': self.P1_V100_MAC,
+                        'eth_dst': arp_mac,
+                        'arp_code': arp.ARP_REQUEST,
+                        'arp_source_ip': '10.0.0.1',
+                        'arp_target_ip': '10.0.0.254'})
+                    # TODO: check ARP reply is valid
+                    self.assertTrue(self.packet_outs_from_flows(arp_replies), msg=arp_mac)
 
         def test_arp_reply_from_host(self):
             """ARP reply for host."""
             arp_replies = self.rcv_packet(1, 0x100, {
                 'eth_src': self.P1_V100_MAC,
                 'eth_dst': FAUCET_MAC,
+                'arp_code': arp.ARP_REPLY,
                 'arp_source_ip': '10.0.0.1',
                 'arp_target_ip': '10.0.0.254'})
             # TODO: check ARP reply is valid
@@ -708,18 +779,22 @@ class ValveTestBases:
 
         def test_nd_for_controller(self):
             """IPv6 ND for controller VIP."""
-            dst_ip = ipaddress.IPv6Address('fc00::1:254')
-            nd_mac = valve_packet.ipv6_link_eth_mcast(dst_ip)
-            ip_gw_mcast = valve_packet.ipv6_solicited_node_from_ucast(dst_ip)
-            nd_replies = self.rcv_packet(2, 0x200, {
-                'eth_src': self.P2_V200_MAC,
-                'eth_dst': nd_mac,
-                'vid': 0x200,
-                'ipv6_src': 'fc00::1:1',
-                'ipv6_dst': str(ip_gw_mcast),
-                'neighbor_solicit_ip': str(dst_ip)})
-            # TODO: check ND reply is valid
-            self.assertTrue(self.packet_outs_from_flows(nd_replies))
+            for dst_ip in (
+                    ipaddress.IPv6Address('fe80::1:254'),
+                    ipaddress.IPv6Address('fc00::1:254')):
+                nd_mac = valve_packet.ipv6_link_eth_mcast(dst_ip)
+                ip_gw_mcast = valve_packet.ipv6_solicited_node_from_ucast(dst_ip)
+                for _retries in range(3):
+                    nd_replies = self.rcv_packet(2, 0x200, {
+                        'eth_src': self.P2_V200_MAC,
+                        'eth_dst': nd_mac,
+                        'vid': 0x200,
+                        'ipv6_src': 'fc00::1:1',
+                        'ipv6_dst': str(ip_gw_mcast),
+                        'neighbor_solicit_ip': str(dst_ip)})
+                    # TODO: check reply NA is valid
+                    packet_outs = self.packet_outs_from_flows(nd_replies)
+                    self.assertTrue(packet_outs)
 
         def test_nd_from_host(self):
             """IPv6 NA from host."""
@@ -760,11 +835,27 @@ class ValveTestBases:
             # TODO: check ping response
             self.assertTrue(self.packet_outs_from_flows(echo_replies))
 
+        def test_unresolved_route(self):
+            """Test unresolved route tries to resolve."""
+            ip_dst = ipaddress.IPv4Network('10.100.100.0/24')
+            ip_gw = ipaddress.IPv4Address('10.0.0.1')
+            valve_vlan = self.valve.dp.vlans[0x100]
+            route_add_replies = self.valve.add_route(
+                valve_vlan, ip_gw, ip_dst)
+            self.assertFalse(route_add_replies)
+            resolve_replies = self.valve.resolve_gateways(
+                time.time(), None)
+            self.assertFalse(resolve_replies)
+            resolve_replies = self.valve.resolve_gateways(
+                time.time() + 99, None)
+            self.assertTrue(resolve_replies)
+
         def test_add_del_route(self):
             """IPv4 add/del of a route."""
             arp_replies = self.rcv_packet(1, 0x100, {
                 'eth_src': self.P1_V100_MAC,
                 'eth_dst': mac.BROADCAST_STR,
+                'arp_code': arp.ARP_REQUEST,
                 'arp_source_ip': '10.0.0.1',
                 'arp_target_ip': '10.0.0.254'})
             # TODO: check ARP reply is valid
@@ -794,6 +885,11 @@ class ValveTestBases:
             # TODO: verify learning rule contents
             # We want to know this host was learned we did not get packet outs.
             self.assertTrue(fib_route_replies)
+            # Verify adding default route via 10.0.0.2
+            self.assertTrue((self.valve.add_route(
+                self.valve.dp.vlans[0x100],
+                ipaddress.IPv4Address('10.0.0.2'),
+                ipaddress.IPv4Network('0.0.0.0/0'))))
             self.assertFalse(self.packet_outs_from_flows(fib_route_replies))
             self.verify_expiry()
 
@@ -1245,7 +1341,7 @@ meters:
         def test_lldp_beacon(self):
             """Test LLDP beacon service."""
             # TODO: verify LLDP packet content.
-            self.assertTrue(self.valve.send_lldp_beacons(time.time(), None))
+            self.assertTrue(self.valve.fast_advertise(time.time(), None))
 
         def test_unknown_port(self):
             """Test port status change for unknown port handled."""
@@ -1334,7 +1430,80 @@ class ValveTestCase(ValveTestBases.ValveTestBig):
     pass
 
 
-class ValveChangePortCase(ValveTestBases.ValveTestSmall):
+class ValveTestEgressPipeline(ValveTestBases.ValveTestBig):
+    """Run complete set of basic tests."""
+
+    DP1_CONFIG = """
+            egress_pipeline: True
+    """ + DP1_CONFIG
+
+
+class ValveFuzzTestCase(ValveTestBases.ValveTestSmall):
+    """Test unknown ports/VLANs."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 0x100
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_fuzz_vlan(self):
+        """Test unknown VIDs/ports."""
+        for i in range(0, 64):
+            self.rcv_packet(1, i, {
+                'eth_src': self.P1_V100_MAC,
+                'eth_dst': self.P2_V200_MAC,
+                'ipv4_src': '10.0.0.2',
+                'ipv4_dst': '10.0.0.3',
+                'vid': i})
+        for i in range(0, 64):
+            self.rcv_packet(i, 0x100, {
+                'eth_src': self.P1_V100_MAC,
+                'eth_dst': self.P2_V200_MAC,
+                'ipv4_src': '10.0.0.2',
+                'ipv4_dst': '10.0.0.3',
+                'vid': 0x100})
+
+
+class ValveDot1xSmokeTestCase(ValveTestBases.ValveTestSmall):
+    """Smoke test to check dot1x can be initialized."""
+
+    CONFIG = """
+dps:
+    s1:
+        hardware: 'GenericTFM'
+%s
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: v100
+                dot1x: true
+            p2:
+                number: 2
+                output_only: True
+vlans:
+    v100:
+        vid: 0x100
+""" % DOT1X_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_get_mac_str(self):
+        """Test NFV port formatter."""
+        self.assertEqual(
+           '00:00:00:0f:01:01',
+           faucet_dot1x.get_mac_str(15, 257))
+
+
+class ValveChangePortTestCase(ValveTestBases.ValveTestSmall):
     """Test changes to config on ports."""
 
     CONFIG = """
@@ -1368,15 +1537,296 @@ dps:
     def setUp(self):
         self.setup_valve(self.CONFIG)
 
-    def test_delete_port(self):
-        """Test port can be deleted."""
+    def test_delete_permanent_learn(self):
+        """Test port permanent learn can deconfigured."""
         self.rcv_packet(2, 0x200, {
             'eth_src': self.P2_V200_MAC,
             'eth_dst': self.P3_V200_MAC,
             'ipv4_src': '10.0.0.2',
             'ipv4_dst': '10.0.0.3',
             'vid': 0x200})
-        self.update_config(self.LESS_CONFIG)
+        self.update_config(self.LESS_CONFIG, reload_type='warm')
+
+
+class ValveDeletePortTestCase(ValveTestBases.ValveTestSmall):
+    """Test deletion of a port."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100]
+            p2:
+                number: 2
+                tagged_vlans: [0x100]
+            p3:
+                number: 3
+                tagged_vlans: [0x100]
+""" % DP1_CONFIG
+
+    LESS_CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100]
+            p2:
+                number: 2
+                tagged_vlans: [0x100]
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_port_delete(self):
+        """Test port can be deleted."""
+        self.update_config(self.LESS_CONFIG, reload_type='cold')
+
+
+class ValveWarmStartVLANTestCase(ValveTestBases.ValveTestSmall):
+    """Test change of port VLAN only is a warm start."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 9
+                tagged_vlans: [0x100]
+            p2:
+                number: 11
+                tagged_vlans: [0x100]
+            p3:
+                number: 13
+                tagged_vlans: [0x100]
+            p4:
+                number: 14
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    WARM_CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 9
+                tagged_vlans: [0x100]
+            p2:
+                number: 11
+                tagged_vlans: [0x100]
+            p3:
+                number: 13
+                tagged_vlans: [0x100]
+            p4:
+                number: 14
+                native_vlan: 0x300
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_warm_start(self):
+        """Test VLAN change is warm startable."""
+        self.update_config(self.WARM_CONFIG, reload_type='warm')
+
+
+class ValveDeleteVLANTestCase(ValveTestBases.ValveTestSmall):
+    """Test deleting VLAN."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100, 0x200]
+            p2:
+                number: 2
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    LESS_CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x200]
+            p2:
+                number: 2
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_delete_vlan(self):
+        """Test VLAN can be deleted."""
+        self.update_config(self.LESS_CONFIG, reload_type='warm')
+
+
+class ValveOFErrorTestCase(ValveTestBases.ValveTestSmall):
+    """Test decoding of OFErrors."""
+
+    def setUp(self):
+        self.setup_valve(CONFIG)
+
+    def test_oferror_parser(self):
+        for type_code, error_tuple in valve_of.OFERROR_TYPE_CODE.items():
+            self.assertTrue(isinstance(type_code, int))
+            type_str, error_codes = error_tuple
+            self.assertTrue(isinstance(type_str, str))
+            for error_code, error_str in error_codes.items():
+                self.assertTrue(isinstance(error_code, int))
+                self.assertTrue(isinstance(error_str, str))
+        test_err = parser.OFPErrorMsg(
+            datapath=None, type_=ofp.OFPET_FLOW_MOD_FAILED, code=ofp.OFPFMFC_UNKNOWN)
+        self.valve.oferror(test_err)
+        test_unknown_type_err = parser.OFPErrorMsg(
+            datapath=None, type_=666, code=ofp.OFPFMFC_UNKNOWN)
+        self.valve.oferror(test_unknown_type_err)
+        test_unknown_code_err = parser.OFPErrorMsg(
+            datapath=None, type_=ofp.OFPET_FLOW_MOD_FAILED, code=666)
+        self.valve.oferror(test_unknown_code_err)
+
+
+class ValveAddVLANTestCase(ValveTestBases.ValveTestSmall):
+    """Test adding VLAN."""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100, 0x200]
+            p2:
+                number: 2
+                tagged_vlans: [0x100]
+""" % DP1_CONFIG
+
+    MORE_CONFIG = """
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                tagged_vlans: [0x100, 0x200]
+            p2:
+                number: 2
+                tagged_vlans: [0x100, 0x300]
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_add_vlan(self):
+        """Test VLAN can added."""
+        self.update_config(self.MORE_CONFIG, reload_type='warm')
+
+
+class ValveChangeACLTestCase(ValveTestBases.ValveTestSmall):
+    """Test changes to ACL on a port."""
+
+    CONFIG = """
+acls:
+    acl_same_a:
+        - rule:
+            actions:
+                allow: 1
+    acl_same_b:
+        - rule:
+            actions:
+                allow: 1
+    acl_diff_c:
+        - rule:
+            actions:
+                allow: 0
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 0x100
+                acl_in: acl_same_a
+            p2:
+                number: 2
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    SAME_CONTENT_CONFIG = """
+acls:
+    acl_same_a:
+        - rule:
+            actions:
+                allow: 1
+    acl_same_b:
+        - rule:
+            actions:
+                allow: 1
+    acl_diff_c:
+        - rule:
+            actions:
+                allow: 0
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 0x100
+                acl_in: acl_same_b
+            p2:
+                number: 2
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    DIFF_CONTENT_CONFIG = """
+acls:
+    acl_same_a:
+        - rule:
+            actions:
+                allow: 1
+    acl_same_b:
+        - rule:
+            actions:
+                allow: 1
+    acl_diff_c:
+        - rule:
+            actions:
+                allow: 0
+dps:
+    s1:
+%s
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 0x100
+                acl_in: acl_diff_c
+            p2:
+                number: 2
+                native_vlan: 0x200
+""" % DP1_CONFIG
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def test_change_port_acl(self):
+        """Test port ACL can be changed."""
+        self.update_config(self.SAME_CONTENT_CONFIG, reload_type='warm')
+        self.update_config(self.DIFF_CONTENT_CONFIG, reload_type='warm')
 
 
 class ValveACLTestCase(ValveTestBases.ValveTestSmall):
@@ -1446,7 +1896,7 @@ acls:
                 self.table.is_output(match, port=3, vid=self.V200),
                 msg='Packet not output before adding ACL')
 
-        self.update_config(acl_config)
+        self.update_config(acl_config, reload_type='cold')
         self.flap_port(2)
         self.assertFalse(
             self.table.is_output(drop_match),
@@ -1475,15 +1925,6 @@ class ValveRootStackTestCase(ValveTestBases.ValveTestSmall):
                 'ipv4_dst': '10.0.0.2'}),
             'vlan_hosts_learned',
             labels={'vlan': str(int(0x300))})
-        self.prom_inc(
-            partial(self.rcv_packet, 5, 0x300, {
-                'eth_src': self.P1_V300_MAC,
-                'eth_dst': self.UNKNOWN_MAC,
-                'vid': 0x300,
-                'ipv4_src': '10.0.0.1',
-                'ipv4_dst': '10.0.0.2'}),
-            'vlan_hosts_learned',
-            labels={'vlan': str(int(0x300))})
 
     def test_stack_flood(self):
         """Test packet flooding when stacking."""
@@ -1529,6 +1970,14 @@ class ValveEdgeStackTestCase(ValveTestBases.ValveTestSmall):
             }]
         self.verify_flooding(matches)
 
+    def test_no_unexpressed_packetin(self):
+        """Test host learning on stack root."""
+        unexpressed_vid = 0x666 | ofp.OFPVID_PRESENT
+        match = {
+            'vlan_vid': unexpressed_vid,
+            'eth_dst': self.UNKNOWN_MAC}
+        self.assertFalse(self.table.is_output(match, port=ofp.OFPP_CONTROLLER, vid=unexpressed_vid))
+
 
 class ValveStackProbeTestCase(ValveTestBases.ValveTestSmall):
     """Test stack link probing."""
@@ -1541,14 +1990,17 @@ dps:
             priority: 1
         interfaces:
             1:
+                description: p1
                 stack:
                     dp: s2
                     port: 1
             2:
+                description: p2
                 stack:
                     dp: s2
                     port: 2
             3:
+                description: p3
                 native_vlan: v100
     s2:
         hardware: 'Open vSwitch'
@@ -1558,25 +2010,31 @@ dps:
             max_per_interval: 1
         interfaces:
             1:
+                description: p1
                 stack:
                     dp: s1
                     port: 1
             2:
+                description: p2
                 stack:
                     dp: s1
                     port: 2
             3:
+                description: p3
                 stack:
                     dp: s3
                     port: 2
             4:
+                description: p4
                 native_vlan: v100
     s3:
         dp_id: 0x3
         interfaces:
             1:
+                description: p1
                 native_vlan: v100
             2:
+                description: p2
                 stack:
                     dp: s2
                     port: 3
@@ -1607,7 +2065,7 @@ vlans:
         other_dp = self.valves_manager.valves[2].dp
         other_port = other_dp.ports[1]
         other_valves = self.valves_manager._other_running_valves(self.valve)
-        self.valve.update_stack_link_states(time.time(), other_valves)
+        self.valve.fast_state_expire(time.time(), other_valves)
         self.assertTrue(stack_port.is_stack_down())
         for change_func, check_func in [
                 ('stack_init', 'is_stack_init'),
@@ -1640,7 +2098,7 @@ vlans:
         self.rcv_lldp(stack_port, other_dp, other_port)
         self.assertTrue(stack_port.is_stack_init())
         other_valves = self.valves_manager._other_running_valves(self.valve)
-        self.valve.update_stack_link_states(time.time() + 300, other_valves) # simulate packet loss
+        self.valve.fast_state_expire(time.time() + 300, other_valves) # simulate packet loss
         self.assertTrue(stack_port.is_stack_down())
 
 
@@ -1667,7 +2125,7 @@ class ValveStackGraphUpdateTestCase(ValveStackProbeTestCase):
             peer_port.stack_down()
             self.valves_manager.valve_flow_services(
                 time.time() + 600,
-                'update_stack_link_states')
+                'fast_state_expire')
             self.assertTrue(port.is_stack_down())
 
         def verify_stack_learn_edges(num_edges, edge=None, test_func=None):
@@ -1715,10 +2173,6 @@ dps:
             p4:
                 number: 4
                 tagged_vlans: [v200]
-            p5:
-                number: 5
-                output_only: True
-                mirror: 4
 vlans:
     v100:
         vid: 0x100
@@ -1852,15 +2306,17 @@ vlans:
 
     def test_lacp(self):
         """Test LACP comes up."""
+        test_port = 1
+        labels = self.port_labels(test_port)
         self.assertEqual(
-            0, int(self.get_prom('port_lacp_status', labels={'port': '1'})))
-        self.rcv_packet(1, 0, {
+            0, int(self.get_prom('port_lacp_status', labels=labels)))
+        self.rcv_packet(test_port, 0, {
             'actor_system': '0e:00:00:00:00:02',
             'partner_system': FAUCET_MAC,
             'eth_dst': slow.SLOW_PROTOCOL_MULTICAST,
             'eth_src': '0e:00:00:00:00:02'})
         self.assertEqual(
-            1, int(self.get_prom('port_lacp_status', labels={'port': '1'})))
+            1, int(self.get_prom('port_lacp_status', labels=labels)))
         self.learn_hosts()
         self.verify_expiry()
 
@@ -1871,7 +2327,7 @@ class ValveReloadConfigTestCase(ValveTestBases.ValveTestBig):
     def setUp(self):
         super(ValveReloadConfigTestCase, self).setUp()
         self.flap_port(1)
-        self.update_config(CONFIG)
+        self.update_config(CONFIG, reload_type='warm', reload_expected=False)
 
 
 class ValveMirrorTestCase(ValveTestBases.ValveTestBig):
